# Generated by Django 5.1.6 on 2025-03-13 14:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('endoreg_db', '0007_rename_pseudo_patient_video_patient_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='reportfile',
            name='patient_examination',
        ),
        migrations.AddField(
            model_name='rawpdffile',
            name='examination',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.patientexamination'),
        ),
        migrations.AddField(
            model_name='rawpdffile',
            name='examiner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.examiner'),
        ),
        migrations.AddField(
            model_name='rawvideofile',
            name='examination',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.patientexamination'),
        ),
        migrations.AddField(
            model_name='reportfile',
            name='examination',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.patientexamination'),
        ),
        migrations.AlterField(
            model_name='reportfile',
            name='examiner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.examiner'),
        ),
        migrations.AlterField(
            model_name='reportfile',
            name='patient',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='endoreg_db.patient'),
        ),
    ]
