# Generated by Django 5.1.6 on 2025-03-13 08:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('endoreg_db', '0005_rawvideofile_frame_count_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='frame',
            name='extracted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='rawframe',
            name='extracted',
            field=models.BooleanField(default=False),
        ),
    ]
