# Generated by Django 5.1.6 on 2025-03-13 12:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('endoreg_db', '0006_frame_extracted_rawframe_extracted'),
    ]

    operations = [
        migrations.RenameField(
            model_name='video',
            old_name='pseudo_patient',
            new_name='patient',
        ),
        migrations.AddField(
            model_name='rawpdffile',
            name='patient',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='raw_pdf_files', to='endoreg_db.patient'),
        ),
    ]
