# Generated by Django 5.1.6 on 2025-03-11 08:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('endoreg_db', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='frame',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=''),
        ),
        migrations.AlterField(
            model_name='rawframe',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=''),
        ),
    ]
