# Generated by Django 5.1.6 on 2025-03-12 06:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('endoreg_db', '0004_alter_rawvideofile_file_alter_video_file'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawvideofile',
            name='frame_count',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='rawvideofile',
            name='state_frames_initialized',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='video',
            name='frame_count',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='video',
            name='state_frames_initialized',
            field=models.BooleanField(default=False),
        ),
    ]
